#include	"includes.h"               /* uC/OS interface */

#include <stdio.h>

/* allocate memory for tasks' stacks */
#ifdef SEMIHOSTED
#define	STACKSIZE	(64+SEMIHOSTED_STACK_NEEDS)
#else
#define	STACKSIZE	64
#endif

/* create stack for tasks */

/* Create two stacks for Task1 and Task2 */

OS_STK Stack1[STACKSIZE];
OS_STK Stack2[STACKSIZE];

/* communication channel */

/*
 * Create two mailbox here.
 * Mbox12 is for Task1 to send message to Task2, and Mbox21 is for Task2
 * to send message back to Task1.
 *
 * Mailbox is defined as type "OS_EVENT", the default value of "OS_MAX_EVENTS" 
 * set in OS_CFG.H is 20. If you have many resources of type OS_EVENT, you
 * have to increase the value of "OS_MAX_EVENTS".
 */
OS_EVENT *Mbox12;
OS_EVENT *Mbox21;

/* data structure for communication */

/*
 * Mail box only passes pointer, so we define a structure here.
 * Then, we use a pointer to structure to pass multivalue from
 * task1 to task2.
 */
typedef struct
{
	char name[64];
	int age;
}envelope;

envelope personal_info;

/*
 * Task2 need to inform Task1 when it completes its job.
 * Task2 passes this token through mailbox to Task1
 */
char token;

/* task1 */
void Task1(void *pata)
{
	INT8U err;

	printf("Task1 called\n");

	for(;;)
    {
		OSMboxPend(Mbox21, 0, &err); // wait for task2 to complete
		
		printf("please enter your name: ");
		scanf("%s", (personal_info.name));

	    do
		{
		  printf("please enter your age: ");
		  scanf("%d", &(personal_info.age));
		}while(personal_info.age < 0);


		OSMboxPost(Mbox12, (void*) &personal_info); // send message to task2
	}
}

void Task2(void *pdata)
{
	INT8U err;
	envelope *info;

	printf("Task2 called\n");

	for(;;)
	{
		info = (envelope*) OSMboxPend(Mbox12, 0, &err); // wait message from task1
	
		printf("\nHello, %s. Nice to meet you!\nYour age is %d.\n\n", info->name, info->age);
		
		OSTimeDly(100);
		OSMboxPost(Mbox21, (void*) &token); // pass a token to task1 on completion
	}
}

/* main function */
int main(int argc, char *argv[])
{
    /* do target (uHAL based ARM system) initialisation */
    ARMTargetInit();

    /* needed by uC/OS */
    OSInit();

	/* create message mailbox */
	Mbox12 = OSMboxCreate((void*) NULL);	// initially, no message from task1 to task2
	Mbox21 = OSMboxCreate((void*) &token);	// initially, a token is in mailbox so that
											// task1 starts running

    /* create the tasks in uC/OS */
    OSTaskCreate(Task1, (void *)0, (void*)&Stack1[STACKSIZE - 1], 4);
    OSTaskCreate(Task2, (void *)0, (void*)&Stack2[STACKSIZE - 1], 3);

    /* Start the (uHAL based ARM system) system running */
    ARMTargetStart();

    /* start the game */
    OSStart();

    /* never reached */
    return(0);
}  